'use strict';

if (!String.prototype.startsWith) {
    String.prototype.startsWith = function (searchString, position) {
        position = position || 0;
        return this.indexOf(searchString, position) === position;
    };
}
if (!String.prototype.includes) {
    String.prototype.includes = function (search, start) {
        'use strict';
        if (typeof start !== 'number') {
            start = 0;
        }

        if (start + search.length > this.length) {
            return false;
        } else {
            return this.indexOf(search, start) !== -1;
        }
    };
}

var Main = function(){
    var vars = {
            displayAlarm: false,
            popupReboot: "<strong>Info: </strong>You need to reboot your Netman 204. Do you want to <a href='#' data-toggle='modal' data-target='#reboot-modal'>REBOOT NOW</a>?",
            popupReload: "<strong>Info: </strong>You need to reload configuration. Do you want to <a href='#' data-toggle='modal' data-target='#restart-modal'>RELOAD NOW</a>?",
            $html: $("html"),
            $body: $("body"),
            $header : $("#header"),
            $loader : $("#page-loader"),
            nominalData : {},
            updateData: {},
            liveData : {},
            netmanData : {},
            sectionData : {},
            panelTypeMap : {
                1: 'panelUPS',
                2: 'panelMDU',
                3: 'panelSTS',
                4: 'panelIRMS',
                5: 'panelMPW',
                6: 'panelUnknown'
            },
            panelConfiguration: 'panelUnknown',
            nominalDataFetchInterval:{},
            liveDataFetchInterval:{},
            netmanDataFetchInterval:{},
            sectionDataFetchInterval:{},
            randomString: "",
            liveDataFailureCnt: 0,
            lastSaveConfigButton: 0,
            dbg: true
        },


      updateViews = function () {

          $(document).attr("title", "Netman 204 - " + vars.nominalData.model + " - " + vars.netmanData.ip_address);
          var prtk = vars.nominalData.PRTK_code;
          if(prtk.substring(0,6) == "GPSER1" ||
              prtk.substring(0,6) == "SENTR1"){
              $("#serial_number").html("Serial number");
              $("#power_kva_label").html("Power [kVA]");
              $("#power_kw_label").html("Power [kW]");
              $("#batt_cap_label").html("Battery capacity [Ah]");
              $("#batt_volt_label").html("Battery voltage [Vdc]");

              if(vars.nominalData.ups_type == 1 ||
                 vars.nominalData.ups_type == 2 ||
                 vars.nominalData.ups_type == 3 ||
                 vars.nominalData.ups_type == 4 ||
                 vars.nominalData.ups_type == 'S') {
                  vars.panelConfiguration = 'panelUPS';
                  if (vars.nominalData.io_conf < 4) {
                      $(".gauge_Load1").attr("y",60);
                      $(".gauge_Load2").addClass("hidden");
                      $(".gauge_Load3").addClass("hidden");
                  }
              }
              else if (vars.nominalData.ups_type == 5 &&
                  !(vars.nominalData.firmware_version.startsWith("SWM040") || vars.nominalData.firmware_version.startsWith("SWM080"))
              ) {
                  vars.panelConfiguration = 'panelUPS';
              }
              else {
                  vars.panelConfiguration = 'panelUnknown';
              }
          }
          else if(prtk.substring(0,6) == "GPSER3"){
              $("#serial_number").html("Serial number");
              $("#power_kva_label").html("Input current [A]");
              $("#power_kw_label").html("-");
              $("#batt_cap_label").html("-");
              $("#batt_volt_label").html("-");

              if(vars.nominalData.ups_type == 7){
                  // si tratta di una MDU
                  vars.panelConfiguration = 'panelMDU';
              }
              else if(vars.nominalData.ups_type == 6){
                  // si tratta di una STS
                  vars.panelConfiguration = 'panelSTS';
              }
              else if (vars.nominalData.ups_type == 5 &&
                  (vars.nominalData.firmware_version.startsWith("SWM040") || vars.nominalData.firmware_version.startsWith("SWM080"))
              ) {
                  vars.panelConfiguration = 'panelSTS';
              }
              else {
                  vars.panelConfiguration = 'panelUnknown';
              }
          }
          else if(prtk.substring(0,6) == "IRSER1") {
              $("#serial_number").html("-");
              $("#power_kva_label").html("Input current [A]");
              $("#power_kw_label").html("-");
              $("#batt_cap_label").html("-");
              $("#batt_volt_label").html("-");

              // ups_type non ha significato
              vars.panelConfiguration = 'panelIRMS';
          }
          else if(prtk.substring(0,6) == "GPSER5") {
              $("#serial_number").html("Serial number");
              $("#power_kva_label").html("Power [kVA]");
              $("#power_kw_label").html("Power [kW]");
              $("#batt_cap_label").html("Battery capacity [Ah]");
              $("#batt_volt_label").html("Battery voltage [Vdc]");

              if (vars.nominalData.ups_type == 'P' ||
                  vars.nominalData.ups_type == 'C' ||
                  vars.nominalData.ups_type == 'A' ||
                  vars.nominalData.ups_type == 'B' ||
                  vars.nominalData.ups_type == 'V') {
                  if (vars.nominalData.force_show_mpw) {
                      vars.nominalData.ups_type = 4;
                      vars.panelConfiguration = 'panelUPS';
                  }
                  else {
                      vars.panelConfiguration = 'panelMPW';
                  }
              }
              else {
                  vars.panelConfiguration = 'panelUnknown';
              }
          }
          else {
              $("#serial_number").html("-");
              $("#power_kva_label").html("-");
              $("#power_kw_label").html("-");
              $("#batt_cap_label").html("-");
              $("#batt_volt_label").html("-");

              vars.panelConfiguration = 'panelUnknown';
          }

          switch (vars.panelConfiguration){
              case 'panelUPS':
                  $("#dashboard-UPS").removeClass("hidden");
                  $("#table-UPS").removeClass("hidden");

                  $("#dashboard-MDU").addClass("hidden");
                  $("#table-MDU").addClass("hidden");

                  $("#dashboard-STS").addClass("hidden");
                  $("#table-STS").addClass("hidden");

                  $("#dashboard-IRMS").addClass("hidden");
                  $("#table-IRMS").addClass("hidden");

                  $("#dashboard-MPW").addClass("hidden");
                  $("#table-MPW").addClass("hidden");

                  $("#dashboard-Unknown").addClass("hidden");
                  $("#table-Unknown").addClass("hidden");

                  vars.$header.find(".tabledata").removeClass("hidden");
                  if (Main.isLoggedIn()) {
                      vars.$header.find(".history").removeClass("hidden");
                  }
                  else {
                      vars.$header.find(".history").addClass("hidden");
                  }

                  if (Main.whoIsLoggedIn() === "power") {
                      $("#tab_Admin_commands").addClass("hidden");
                  }
                  else {
                      $("#tab_Admin_commands").removeClass("hidden");
                  }
                  $("#tab-command-testbattery").removeClass("hidden");
                  if (prtk.substring(0,6) == "SENTR1") {
                      $("#tab-command-onbypass").removeClass("hidden");
                  }
                  $("#tab-command-shutdown").removeClass("hidden");
                  $("#tab-command-restore").removeClass("hidden");
                  break;
              case 'panelMDU':
                  $("#dashboard-UPS").addClass("hidden");
                  $("#table-UPS").addClass("hidden");

                  $("#dashboard-MDU").removeClass("hidden");
                  $("#table-MDU").removeClass("hidden");

                  $("#dashboard-STS").addClass("hidden");
                  $("#table-STS").addClass("hidden");

                  $("#dashboard-IRMS").addClass("hidden");
                  $("#table-IRMS").addClass("hidden");

                  $("#dashboard-MPW").addClass("hidden");
                  $("#table-MPW").addClass("hidden");

                  $("#dashboard-Unknown").addClass("hidden");
                  $("#table-Unknown").addClass("hidden");

                  vars.$header.find(".tabledata").removeClass("hidden");
                  if (Main.isLoggedIn()) {
                      vars.$header.find(".history").removeClass("hidden");
                  }
                  else {
                      vars.$header.find(".history").addClass("hidden");
                  }
                  $("#tab_history_datalog").addClass("hidden");
                  if (Main.whoIsLoggedIn() === "power") {
                      $("#tab_Admin_commands").addClass("hidden");
                  }
                  else {
                      $("#tab_Admin_commands").removeClass("hidden");
                  }
                  $("#tab-command-shutdown-mdu").removeClass("hidden");
                  $("#tab-command-restore-mdu").removeClass("hidden");
                  $("#tab-command-reboot-mdu").removeClass("hidden");
                  break;
              case 'panelSTS':
                  $("#dashboard-UPS").addClass("hidden");
                  $("#table-UPS").addClass("hidden");

                  $("#dashboard-MDU").addClass("hidden");
                  $("#table-MDU").addClass("hidden");

                  $("#dashboard-STS").removeClass("hidden");
                  $("#table-STS").removeClass("hidden");

                  $("#dashboard-IRMS").addClass("hidden");
                  $("#table-IRMS").addClass("hidden");

                  $("#dashboard-MPW").addClass("hidden");
                  $("#table-MPW").addClass("hidden");

                  $("#dashboard-Unknown").addClass("hidden");
                  $("#table-Unknown").addClass("hidden");

                  vars.$header.find(".tabledata").removeClass("hidden");
                  if (Main.isLoggedIn()) {
                      vars.$header.find(".history").removeClass("hidden");
                  }
                  else {
                      vars.$header.find(".history").addClass("hidden");
                  }
                  $("#tab_history_datalog").addClass("hidden");
                  //if (Main.whoIsLoggedIn() === "power") {
                      $("#tab_Admin_commands").addClass("hidden");
                  //}
                  //else {
                  //    $("#tab_Admin_commands").removeClass("hidden");
                  //}
                  $("#tab-command-shutdown-xts").removeClass("hidden");
                  $("#tab-command-restore-xts").removeClass("hidden");
                  $("#tab-command-reboot-xts").removeClass("hidden");
                  break;
              case 'panelIRMS':
                  $("#dashboard-UPS").addClass("hidden");
                  $("#table-UPS").addClass("hidden");

                  $("#dashboard-MDU").addClass("hidden");
                  $("#table-MDU").addClass("hidden");

                  $("#dashboard-STS").addClass("hidden");
                  $("#table-STS").addClass("hidden");

                  $("#dashboard-IRMS").removeClass("hidden");
                  $("#table-IRMS").removeClass("hidden");

                  $("#dashboard-MPW").addClass("hidden");
                  $("#table-MPW").addClass("hidden");

                  $("#dashboard-Unknown").addClass("hidden");
                  $("#table-Unknown").addClass("hidden");

                  vars.$header.find(".tabledata").removeClass("hidden");
                  if (Main.isLoggedIn()) {
                      vars.$header.find(".history").removeClass("hidden");
                  }
                  else {
                      vars.$header.find(".history").addClass("hidden");
                  }

                  $("#tab_history_datalog").addClass("hidden");
                  if (Main.whoIsLoggedIn() === "power") {
                      $("#tab_Admin_commands").addClass("hidden");
                  }
                  else {
                      $("#tab_Admin_commands").removeClass("hidden");
                  }
                  $("#tab-command-shutdown-irms").removeClass("hidden");
                  $("#tab-command-restore-irms").removeClass("hidden");
                  $("#tab-command-reboot-irms").removeClass("hidden");
                  $("#tab-command-switch-irms").removeClass("hidden");
                  break;
              case 'panelMPW':
                  $("#dashboard-UPS").addClass("hidden");
                  $("#table-UPS").addClass("hidden");

                  $("#dashboard-MDU").addClass("hidden");
                  $("#table-MDU").addClass("hidden");

                  $("#dashboard-STS").addClass("hidden");
                  $("#table-STS").addClass("hidden");

                  $("#dashboard-IRMS").addClass("hidden");
                  $("#table-IRMS").addClass("hidden");

                  $("#dashboard-MPW").removeClass("hidden");
                  $("#table-MPW").removeClass("hidden");

                  $("#dashboard-Unknown").addClass("hidden");
                  $("#table-Unknown").addClass("hidden");

                  vars.$header.find(".tabledata").addClass("hidden");
                  vars.$header.find(".history").addClass("hidden");
                  $("#tab_Admin_commands").addClass("hidden");
                  break;
              default:
              case 'panelUnknown':
                  $("#dashboard-UPS").addClass("hidden");
                  $("#table-UPS").addClass("hidden");

                  $("#dashboard-MDU").addClass("hidden");
                  $("#table-MDU").addClass("hidden");

                  $("#dashboard-STS").addClass("hidden");
                  $("#table-STS").addClass("hidden");

                  $("#dashboard-IRMS").addClass("hidden");
                  $("#table-IRMS").addClass("hidden");

                  $("#dashboard-MPW").addClass("hidden");
                  $("#table-MPW").addClass("hidden");

                  $("#dashboard-Unknown").removeClass("hidden");
                  $("#table-Unknown").removeClass("hidden");

                  vars.$header.find(".tabledata").addClass("hidden");
                  vars.$header.find(".history").addClass("hidden");
                  $("#tab_Admin_commands").addClass("hidden");
                  break;
          }
      },
    // ---- Live data requests and handling

        fetchFirmwareUpdateAvailable = function fetchFirmwareUpdateAvailable(){
            //per renderlo configurabile dovremmo impostarlo sul config.js; tuttavia questo viene chiesto solo quando
            //si configura (con utente admin). Se non siamo admin non possiamo accedere, altrimenti tutti vedono le
            //password (problema di sicurezza).
            //Ho lasciato l'if ma non c'e' una soluzione pulita e accettabile
        //if (false)
            // fetches update information from website and then updates the UI.
            return $.ajax({
                dataType: "json",
                url: Config.get("updateAvailable"),
                cache: false,
                timeout: 1500
            }).done( function(data){
                vars.updateData = deepTrim(data);
                updateNominalData();
            }).error(function(xhr, status, error){
                //var err = eval("(" + xhr.responseText + ")");
                //alert(err);
            });
        },

        fetchNominalData = function fetchNominalData(){
            // fetches files from Nominal Data file and then updates the UI.
            return $.ajax({
                dataType: "json",
                url: Config.get("nominalDataFileURL"),
                cache: false
            }).done( function(data){
                vars.nominalData = deepTrim(data);
                updateViews();
                updateNominalData();
            }).error(function(){
                vars.displayAlarm=true;
            });
        },

        fetchLiveData = function fetchLiveData(){
            // fetches files from Live Data file and then updates the UI.
            return $.ajax({
                dataType: "json",
                url: Config.get("liveDataFileURL"),
                cache: false
            }).done(function(data){
                vars.liveData = deepTrim(data);
                updateLiveData();
                if( vars.panelConfiguration == 'panelUnknown' ) {
                    // qualcosa non torna nel prtk_code e ups_type
                    // sovrascrivo stato e allarmi ricevuti con scritte adeguate
                    if (vars.liveData.alarms[0] === undefined || !vars.liveData.alarms[0].alarm.toUpperCase().startsWith("COMMUNICATIONS LOST") ) {
                        vars.liveData.system_status.status = 'UNKNOW';
                        vars.liveData.alarms = [];
                        vars.liveData.alarms[0] = {category: 1, alarm: "WRONG PRTK CODE"};
                    }
                    if (vars.displayAlarm){
                        vars.liveData.system_status.status = 'UNKNOW';
                        vars.liveData.alarms = [];
                        vars.liveData.alarms[0] = {category: 0, alarm: "APPLICATION ERROR"};
                    }
                }
                updateAlarmList();

                //only for LiveData handle the IsAlive status
                handleIsAlive(true);
            }).error(function(){
                // se e' attiva la finestra modale del reboot non visualizzare la perdita di comunicazione.
                if( $("#rebooting-modal").hasClass('in'))
                    handleIsAlive(true);    // non visualizzare la modale del riavvio applicativo.
                else
                    handleIsAlive(false);
            });
        },

        fetchNetmanData = function fetchNetmanData(){
            return $.ajax({
                dataType: "json",
                url: Config.get("netmanDataFileURL"),
                cache: false
            }).done(function(data){
                vars.netmanData = deepTrim(data);
                updateNetmanData();
            });
        },

        fetchSectionData = function fetchSectionData(){
            return $.ajax({
                dataType: "json",
                url: Config.get("refreshTokenFileURL"),
                cache: false
            }).done(function(data){
                vars.sectionData = deepTrim(data);
                updateSectionData();
            });
        },

        handleIsAlive = function handleIsAlive(isAlive) {

            var $isDeadModal = $('#isdead-modal');

            if ((moment(new Date()).format("X") - vars.lastSaveConfigButton)  < 45) {
                // si tratta di un riavvio applicativo previsto... cambio il messaggio della modale
                $isDeadModal.find(".modal-title").html( "Please wait..." );
                $isDeadModal.find(".modal-body").html( "Applying new settings" );
            } else {
                $isDeadModal.find(".modal-title").html( "Your Netman 204 is probably not connected, off or on a different IP address." );
                $isDeadModal.find(".modal-body").html( "Control if your Netman 204 is correctly plugged in, turned on and the network is correctly configured." );
            }

            if (isAlive) {
                $isDeadModal.modal('hide');
                vars.liveDataFailureCnt = 0;
            } else {
                vars.liveDataFailureCnt += 1;
                if(vars.liveDataFailureCnt >= 3)
                    $isDeadModal.modal({backdrop: true, keyboard: false});
            }

        },

        updateLiveData = function updateLiveData(){

            checkReboot();

            // updates live data
            switch(vars.$body.data("page")){
                case "dashboard":
                    switch (vars.panelConfiguration) {
                        case 'panelUPS':
                            Dashboard.setUPSStatus(vars.liveData.system_status.sinottico);
                            Dashboard.updateUPSVariables(vars.liveData);
                            Dashboard.setUPSIconsColor(vars.liveData.system_status);
                            Dashboard.setGaugeValue(vars.liveData);
                            break;
                        case 'panelMDU':
                            Dashboard.setMDUStatus(".icons .icon_output1", "system_output1", vars.liveData.sout1);
                            Dashboard.setMDUStatus(".icons .icon_output2", "system_output2", vars.liveData.sout2);
                            Dashboard.setMDUStatus(".icons .icon_output3", "system_output3", vars.liveData.sout3);
                            Dashboard.setMDUStatus(".icons .icon_output4", "system_output4", vars.liveData.sout4);
                            Dashboard.setMDUStatus(".icons .icon_output5", "system_output5", vars.liveData.sout5);
                            Dashboard.setMDUStatus(".icons .icon_output6", "system_output6", vars.liveData.sout6);
                            Dashboard.setMDUStatus(".icons .icon_output7", "system_output7", vars.liveData.sout7);
                            Dashboard.setMDUStatus(".icons .icon_output8", "system_output8", vars.liveData.sout8);
                            Dashboard.updateMDUVariables(vars.liveData);
                            Dashboard.setMDUIconsColor(vars.liveData.system_status);
                            break;
                        case 'panelSTS':
                            if (Main.isATS()) {
                                $("#SWMB1toOUT").addClass("hidden");
                                $("#IN1toSWMB1").addClass("hidden");
                                $(".icon_SWMB1").addClass("hidden");
                                $("#SWMB2toOUT").addClass("hidden");
                                $("#IN2toSWMB2").addClass("hidden");
                                $(".icon_SWMB2").addClass("hidden");
                            }
                            else {
                                $("#SWMB1toOUT").removeClass("hidden");
                                $("#IN1toSWMB1").removeClass("hidden");
                                $(".icon_SWMB1").removeClass("hidden");
                                $("#SWMB2toOUT").removeClass("hidden");
                                $("#IN2toSWMB2").removeClass("hidden");
                                $(".icon_SWMB2").removeClass("hidden");
                            }
                            if ( vars.liveData.system_status.preferredsource == 1 ) {
                                $(".preferredSource1").removeClass("hidden");
                                $(".preferredSource2").addClass("hidden");
                            }
                            else {
                                $(".preferredSource1").addClass("hidden");
                                $(".preferredSource2").removeClass("hidden");
                            }
                            Dashboard.setSTSStatus(".icons .icon_input1", "SWMB1toOUT", (vars.liveData.system_status.switchbypass1 == 1) ? vars.liveData.system_status.source1_color : "#A7A7A7");
                            Dashboard.setSTSStatus(".icons .icon_input1", "IN1toSWIN1", vars.liveData.system_status.source1_color);
                            Dashboard.setSTSStatus(".icons .icon_input1", "IN1toSWMB1", vars.liveData.system_status.source1_color);

                            Dashboard.setSTSStatus(".icons .icon_input2", "SWMB2toOUT", (vars.liveData.system_status.switchbypass2 == 1) ? vars.liveData.system_status.source2_color : "#A7A7A7");
                            Dashboard.setSTSStatus(".icons .icon_input2", "IN2toSWIN2", vars.liveData.system_status.source2_color);
                            Dashboard.setSTSStatus(".icons .icon_input2", "IN2toSWMB2", vars.liveData.system_status.source2_color);

                            Dashboard.setSTSStatus(".icons .icon_system1", "SWIN1toSWOUT_up", vars.liveData.system_status.system1_color);
                            Dashboard.setSTSStatus(".icons .icon_system1", "SWIN1toSWOUT_dwn", vars.liveData.system_status.system1_color);
                            if ( vars.liveData.system_status.onsource == 1 )
                                $("#SWIN1toSWOUT_up").removeClass("hidden");
                            else
                                $("#SWIN1toSWOUT_up").addClass("hidden");

                            Dashboard.setSTSStatus(".icons .icon_system2", "SWIN2toSWOUT", vars.liveData.system_status.system2_color);

                            Dashboard.setSTSStatus(".icons .icon_output", "SWOUTtoOUT", vars.liveData.system_status.output_color);

                            Dashboard.updateSTSVariables(vars.liveData);
                            Dashboard.setSTSSwitchStatus(".icon_SWIN1", vars.liveData.system_status.switchinput1 );
                            Dashboard.setSTSSwitchStatus(".icon_SWIN2", vars.liveData.system_status.switchinput2 );
                            Dashboard.setSTSSwitchStatus(".icon_SWMB1", vars.liveData.system_status.switchbypass1 );
                            Dashboard.setSTSSwitchStatus(".icon_SWMB2", vars.liveData.system_status.switchbypass2 );
                            Dashboard.setSTSSwitchStatus(".icon_SWOUT", vars.liveData.system_status.switchoutput );
                            break;
                        case 'panelIRMS':
                            Dashboard.setIRMSStatus(".icons .icon_output1", "system_output1", (vars.liveData.sourceout1 == 0 ? vars.liveData.system_status.system1_color : vars.liveData.system_status.system2_color), vars.liveData.sout1);
                            Dashboard.setIRMSStatus(".icons .icon_output2", "system_output2", (vars.liveData.sourceout2 == 0 ? vars.liveData.system_status.system1_color : vars.liveData.system_status.system2_color), vars.liveData.sout2);
                            Dashboard.setIRMSStatus(".icons .icon_output3", "system_output3", (vars.liveData.sourceout3 == 0 ? vars.liveData.system_status.system1_color : vars.liveData.system_status.system2_color), vars.liveData.sout3);
                            Dashboard.setIRMSStatus(".icons .icon_output4", "system_output4", (vars.liveData.sourceout4 == 0 ? vars.liveData.system_status.system1_color : vars.liveData.system_status.system2_color), vars.liveData.sout4);
                            Dashboard.setIRMSStatus(".icons .icon_output5", "system_output5", (vars.liveData.sourceout5 == 0 ? vars.liveData.system_status.system1_color : vars.liveData.system_status.system2_color), vars.liveData.sout5);
                            Dashboard.setIRMSStatus(".icons .icon_output6", "system_output6", (vars.liveData.sourceout6 == 0 ? vars.liveData.system_status.system1_color : vars.liveData.system_status.system2_color), vars.liveData.sout6);
                            Dashboard.setIRMSStatus(".icons .icon_output7", "system_output7", (vars.liveData.sourceout7 == 0 ? vars.liveData.system_status.system1_color : vars.liveData.system_status.system2_color), vars.liveData.sout7);
                            Dashboard.setIRMSStatus(".icons .icon_output8", "system_output8", (vars.liveData.sourceout8 == 0 ? vars.liveData.system_status.system1_color : vars.liveData.system_status.system2_color), vars.liveData.sout8);
                            Dashboard.updateIRMSVariables(vars.liveData);
                            Dashboard.setIRMSIconsColor(vars.liveData.system_status);
                            break;
                        case 'panelMPW':
                        default:
                        case 'panelUnknown':
                            break;
                    }

                    initSensorsData();

                    break;
                case "monitor":
                    switch (vars.panelConfiguration) {
                        case 'panelUPS':
                            Monitor.setUPSStatus(vars.liveData.system_status.sinottico);
                            Monitor.updateUPSVariables(vars.liveData);
                            Monitor.updateAlarmList(vars.liveData);
                            break;
                        case 'panelMDU':
                            break;
                        case 'panelSTS':
                            break;
                        case 'panelIRMS':
                            break;
                        case 'panelMPW':
                        default:
                        case 'panelUnknown':
                            break;
                    }
                    break;
                case "tabledata":
                    switch (vars.panelConfiguration) {
                        case 'panelUPS':
                            TableData.updateUPSVariables(vars.liveData);
                            break;
                        case 'panelMDU':
                            TableData.updateMDUVariables(vars.liveData);
                            break;
                        case 'panelSTS':
                            TableData.updateSTSVariables(vars.liveData);
                            break;
                        case 'panelIRMS':
                            TableData.updateIRMSVariables(vars.liveData);
                            break;
                        case 'panelMPW':
                        default:
                        case 'panelUnknown':
                            break;
                    }
                    initSensorsData();
                    break;
                case "system-overview":
                    var $network = $(".measurements-block .network");

                    $network.find(".networkCurrentDate").html( nullReplace( vars.liveData.current_date ) );
                    break;
                case "configuration":
                    $(".currentDate").html( nullReplace( vars.liveData.current_date ) );
                    break;
            }
        },

        updateNominalData = function updateNominalData(){

            vars.$header.find(".ups-model").html(nullReplace(vars.nominalData.model));

            switch(vars.$body.data("page")) {
                case "system-overview":
                    var
                        $device = $(".measurements-block .device"),
                        $deviceConf = $(".measurements-block .device-conf");

                    $device.find(".deviceModel").html(nullReplace(vars.nominalData.model));
                    if ((vars.panelConfiguration == 'panelUPS') || (vars.panelConfiguration == 'panelMPW')) {
                        $device.find(".deviceSerialNumber").html( nullReplace( vars.nominalData.serial_number ) );
                        $device.find(".devicePowerKva").html(nullReplace(vars.nominalData.power_kva));
                        $device.find(".devicePowerKw").html(nullReplace(vars.nominalData.power_kw));
                        $device.find(".deviceBatteryCapacity").html( nullReplace( vars.nominalData.battery_capacity ) );
                        $device.find(".deviceBatteryVoltage").html( nullReplace( vars.nominalData.battery_voltage ) );
                    }
                    else {
                        if (vars.panelConfiguration == 'panelIRMS') {
                            $device.find(".deviceSerialNumber").html("-");
                        }
                        else
                        {
                            $device.find(".deviceSerialNumber").html( nullReplace( vars.nominalData.serial_number ) );
                        }
                        $device.find(".devicePowerKva").html(nullReplace(vars.nominalData.current));
                        $device.find(".devicePowerKw").html(nullReplace(vars.nominalData.power_kw));
                        $device.find(".deviceBatteryCapacity").html( nullReplace( vars.nominalData.battery_capacity ) );
                        $device.find(".deviceBatteryVoltage").html( nullReplace( vars.nominalData.battery_voltage ) );
                    }
                    $device.find(".deviceFirmwareVersion").html( nullReplace( vars.nominalData.firmware_version ) );

                    $deviceConf.find(".devicePRTKcode").html( nullReplace( vars.nominalData.PRTK_code ) );
                    //$deviceConf.find(".deviceAddress").html( nullReplace( vars.nominalData.Address ) ); tolto
                    $deviceConf.find(".deviceName").html( nullReplace( vars.nominalData.Name ) );
                    break;

                case "dashboard":
                    break;

                case "monitor":
                    vars.$header.find(".devicePowerKva").html(nullReplace(vars.nominalData.power_kva));
                    vars.$header.find(".devicePowerKw").html(nullReplace(vars.nominalData.power_kw));
                    break;

                case "configuration":
                    var $network = $(".device-general-config");
                    $network.find(".factorySerialNumber").html( nullReplace( vars.nominalData.factory_serial_number ) );
                    break;
            }

        },

        updateNetmanData = function updateNetmanData(){

            switch(vars.$body.data("page")) {
                case "system-overview":
                    var $network = $(".measurements-block .network"),
                        $networkConf = $(".measurements-block .network-conf");

                    if(!vars.netmanData.debug){
                        $network.find(".debugMode").hide();
                    }

                    $network.find(".networkCardVersion").html( nullReplace( vars.netmanData.card_version ) );
                    $network.find(".networkSerialNumber").html( nullReplace( vars.netmanData.serial_number ) );
                    $network.find(".networkMACAddress").html( nullReplace( vars.netmanData.mac_address ) );
                    $network.find(".networkAppVersion").html( nullReplace( vars.netmanData.application_version ) );

                    $network.find(".networkAppVersion").attr('data-original-title', "java: " + vars.netmanData.java_version +
                                                                                    "<br> harness: " + vars.netmanData.harness_version +
                                                                                    "<br> web: " + vars.netmanData.web_version);
                    $network.find(".networkSysVersion").html( nullReplace( vars.netmanData.system_version ) );

                    if(vars.netmanData.official){
                        $network.find(".networkAppVersion-official").addClass('is-official').attr('data-original-title', 'official');
                    }else{
                        $network.find(".networkAppVersion-official").removeClass('is-official').attr('data-original-title', 'unofficial');
                    }

                    $network.find(".networkKernel").html( nullReplace( vars.netmanData.kernel ) );

                    $networkConf.find(".networkHostname").html( nullReplace( vars.netmanData.hostname ) );
                    $networkConf.find(".networkDHCP").html( nullReplace( vars.netmanData.dhcp_enabled ) );
                    $networkConf.find(".networkIP").html( nullReplace( vars.netmanData.ip_address ) );
                    $networkConf.find(".networkIPv6").html( nullReplace( vars.netmanData.ip_address_v6 ) );
                    $networkConf.find(".networkNetmask").html( nullReplace( vars.netmanData.netmask ) );

                    $networkConf.find(".networkGateway").html( nullReplace( vars.netmanData.gateway ) );
                    $networkConf.find(".networkDns1").html( nullReplace( vars.netmanData.dns1 ) );
                    $networkConf.find(".networkDns2").html( nullReplace( vars.netmanData.dns2 ) );
                    break;
                case "recoverpassword":
                    $("#recovery-instruction").html("  1)  Please send via mail to service this code: <strong>204:" + vars.netmanData.mac_address + ":" + vars.netmanData.serial_number + "</strong>");
                    $("#recovery-instruction2").html("  2)  Submit the <strong>RECOVERY CODE</strong> received via mail by the service in the form below");
                    break;
                case "administration":
                    $("#netman-administration").find(".networkAppVersion").html( nullReplace( vars.netmanData.application_version ) );
                    $("#netman-administration").find(".networkSysVersion").html( nullReplace( vars.netmanData.system_version ) );
                    break;
            }
        },

        updateSectionData = function updateSectionData(){

            if( isLoggedIn() ) {
                //logged in
                vars.$header.find(".user-name").find(".name").text( showWhoisLoggedIn() );
                vars.$header.find(".user-drop").find(".logged-in").show();
                vars.$header.find(".user-drop").find(".not-logged-in").hide();
                switch(whoIsLoggedIn()){
                    case "admin":
                        vars.$header.find(".user-image").addClass("is-admin");
                        vars.$header.find(".user-image").removeClass("is-power");
                        break;
                    case "power":
                        vars.$header.find(".user-image").removeClass("is-admin");
                        vars.$header.find(".user-image").addClass("is-power");
                        break;
                }
                $(".download-conf").removeClass("hidden");
                vars.$header.find(".system-overview").removeClass("hidden");
                vars.$header.find(".configuration").removeClass("hidden");
                vars.$header.find(".administration").removeClass("hidden");
            } else {
                //not logged in
                //Cookies.remove("token");
                vars.$header.find(".user-name").find(".name").text("User");
                vars.$header.find(".user-drop").find(".not-logged-in").show();
                vars.$header.find(".user-drop").find(".logged-in").hide();
                vars.$header.find(".user-image").removeClass("is-admin");
                vars.$header.find(".user-image").removeClass("is-power");
                $(".download-conf").addClass("hidden");
                vars.$header.find(".system-overview").addClass("hidden");
                vars.$header.find(".configuration").addClass("hidden");
                vars.$header.find(".administration").addClass("hidden");
            }
            if (vars.sectionData.login_mode === "local") {
                $("#loginwith").text("login with local authentication");
                $("#logintype").prop("selectedIndex" , "0");
                $("#logintype").hide();
            }
            if (vars.sectionData.login_mode === "ldap") {
                $("#loginwith").text("login with LDAP authentication");
                $("#logintype").prop("selectedIndex" , "1");
                $("#logintype").hide();
            }
        },

        updateAlarmList = function updateAlarmList(){

            if ((vars.updateData != undefined) && (vars.updateData.latest_app != undefined) &&
                (vars.netmanData.application_version.startsWith("03")) &&
                (vars.netmanData.application_version.length === 5) &&
                (vars.netmanData.application_version != vars.updateData.latest_app)
            ) {
                var local_version = parseInt(vars.netmanData.application_version.substring(3));
                var web_version = parseInt(vars.updateData.latest_app.substring(3));
                if (web_version > local_version) {
                    vars.liveData.alarms.push({
                        category: 4,
                        alarm: "update available " + vars.updateData.latest_app
                    });
                }
            }

            if ((vars.updateData != undefined) && (vars.updateData.latest_os != undefined) && (vars.netmanData.system_version != vars.updateData.latest_os))
                vars.liveData.alarms.push({
                    category: 4,
                    alarm: "System update available " + vars.updateData.latest_os
                });
            var $widgetWrapperAlarm = $(".js-alarms-widgets"),
                alarmTpls = {
                    'row' : [
                        "<li class='alarm-row'><a class='type'>---</a></li>",
                    ].join(" ")
                },
                i,
                tpl,
                alarms = vars.liveData.alarms,
                status_color = vars.liveData.system_status.status_color;

            //checking alarms
            $widgetWrapperAlarm.empty();

            if(alarms && alarms.length > 0){
                vars.$header.find(".alarm-name").find(".name").text(alarms[0].alarm);
                vars.$header.find(".alarm-box").find(".badge").removeClass("hidden");
                vars.$header.find(".alarm-box").find(".js-alarms-widgets").removeClass("hidden");
                vars.$header.find(".alarm-box").find(".badge").text(alarms.length);

                for(i=0; i<alarms.length; i++){

                    if(alarms[i] !== undefined){

                        var sensorType;

                        sensorType = 'row';

                        tpl = $(alarmTpls[sensorType]).clone();

                        var ref = tpl.find(".type");
                        switch (alarms[i].category){
                            default:
                            case 0:
                                ref.addClass("commlost");
                                if( i == 0) {
                                    status_color = "#777777";
                                    vars.$header.find(".alarm-box").find(".badge").addClass("commlost");
                                    vars.$header.find(".alarm-box").find(".badge").removeClass("info");
                                    vars.$header.find(".alarm-box").find(".badge").removeClass("minor");
                                    vars.$header.find(".alarm-box").find(".badge").removeClass("major");
                                    vars.$header.find(".alarm-box").find(".badge").removeClass("critical");
                                }
                                break;
                            case 1:
                                ref.addClass("critical");
                                if( i == 0) {
                                    status_color = "#cc0000";
                                    vars.$header.find(".alarm-box").find(".badge").addClass("critical");
                                    vars.$header.find(".alarm-box").find(".badge").removeClass("info");
                                    vars.$header.find(".alarm-box").find(".badge").removeClass("minor");
                                    vars.$header.find(".alarm-box").find(".badge").removeClass("major");
                                    vars.$header.find(".alarm-box").find(".badge").removeClass("commlost");
                                }
                                break;
                            case 2:
                                ref.addClass("major");
                                if( i == 0) {
                                    status_color = "#e94f0f";
                                    vars.$header.find(".alarm-box").find(".badge").addClass("major");
                                    vars.$header.find(".alarm-box").find(".badge").removeClass("info");
                                    vars.$header.find(".alarm-box").find(".badge").removeClass("minor");
                                    vars.$header.find(".alarm-box").find(".badge").removeClass("commlost");
                                    vars.$header.find(".alarm-box").find(".badge").removeClass("critical");
                                }
                                break;
                            case 3:
                                ref.addClass("minor");
                                if( i == 0) {
                                    status_color = "#f0ad4e";
                                    vars.$header.find(".alarm-box").find(".badge").addClass("minor");
                                    vars.$header.find(".alarm-box").find(".badge").removeClass("info");
                                    vars.$header.find(".alarm-box").find(".badge").removeClass("commlost");
                                    vars.$header.find(".alarm-box").find(".badge").removeClass("major");
                                    vars.$header.find(".alarm-box").find(".badge").removeClass("critical");
                                }
                                break;
                            case 4:
                                ref.addClass("info");
                                if( i == 0) {
                                    vars.$header.find(".alarm-box").find(".badge").addClass("info");
                                    vars.$header.find(".alarm-box").find(".badge").removeClass("commlost");
                                    vars.$header.find(".alarm-box").find(".badge").removeClass("minor");
                                    vars.$header.find(".alarm-box").find(".badge").removeClass("major");
                                    vars.$header.find(".alarm-box").find(".badge").removeClass("critical");
                                }
                                break;
                        }
                        ref.text(alarms[i].alarm);

                        tpl.appendTo($widgetWrapperAlarm);
                    }
                }
            } else {
                vars.$header.find(".alarm-name").find(".name").text("None");
                vars.$header.find(".alarm-box").find(".badge").addClass("hidden");
                vars.$header.find(".alarm-box").find(".js-alarms-widgets").addClass("hidden");
            }

            // ora posso colorare e scrive il system status
            vars.$header.find(".systemStatus")
                .html( nullReplace( vars.liveData.system_status.status ) )
                .css( "color", status_color );

        },

        clearIntervals = function clearIntervals(){
            window.clearInterval(vars.liveDataFetchInterval);
            window.clearInterval(vars.nominalDataFetchInterval);
            window.clearInterval(vars.netmanDataFetchInterval);
            window.clearInterval(vars.sectionDataFetchInterval);
            Dashboard.clearIntervals();
        },

    // ----

        isLoggedIn = function isLoggedIn(){
            return (vars.sectionData.active_session === true && Cookies.get("token") !== undefined );
        },

        whoIsLoggedIn = function whoIsLoggedIn(){
            if (vars.sectionData.current_user === "LDAP")
                return "power";
            return (vars.sectionData.current_user );
        },

        showWhoisLoggedIn = function showWhoisLoggedIn(){
            return (vars.sectionData.current_user );
        },

        isLocalLoginAvailable = function isLocalLoginAvailable() {
            return (vars.sectionData.login_mode === "local") || (vars.sectionData.login_mode === "both");
        },


        isLDAPLoginAvailable = function isLDAPLoginAvailable() {
            return (vars.sectionData.login_mode === "ldap") || (vars.sectionData.login_mode === "both");
        },

        OSVersion = function OSVersion(){
            return (vars.netmanData.system_version);
        },

        doLogout = function doLogout(){
            var logout = $.ajax({
                url: 'cgi-bin/logout.cgi'
            }).done(function(data){
                var res = JSON.parse(data);

                if(res.response === "ok"){
                    setSuccess(Config.get("logoutSuccessMessage"));
                    Cookies.remove('token');
                    fetchNetmanData();
                }
                window.location.href = 'login.html';
            });
        },

        initForms = function($context){
            if($context === undefined){ $context = $("#main"); }
            var $forms = $context.find("form");

            if (Main.whoIsLoggedIn() === "power") {
                $("#change_pwd_admin").addClass("hidden");
                $("#revoke_power_user").addClass("hidden");
                $("#power_user_description").addClass("hidden");
                $("#tab_loginaccess").addClass("hidden")
                $("#tab_resetdefault").addClass("hidden")
            }
            else {
                $("#change_pwd_admin").removeClass("hidden");
                $("#revoke_power_user").removeClass("hidden");
                $("#power_user_description").removeClass("hidden");
                $("#tab_loginaccess").removeClass("hidden")
                $("#tab_resetdefault").removeClass("hidden")
            }
            if (vars.sectionData.local_user === true) {
                $("#tab_changepassword").removeClass("hidden");
            } else {
                if ((Main.whoIsLoggedIn() === "admin") && (vars.sectionData.login_mode !== "ldap")) {
                    $("#tab_changepassword").removeClass("hidden");
                }
            }
            $.each($forms, function(i,el){
                var $form = $(el);
                $form.submit(function(e){e.preventDefault();});
                if($form.hasClass('admin-password')){
                    $form.validate({
                        submitHandler : submitPasswordHandler,
                        rules: {
                            admin_password_confirm: {equalTo: '#admin_password'}
                        }
                    });
                }
                else if($form.hasClass('power-password')){
                    $form.validate({
                        submitHandler : submitPasswordHandler,
                        rules: {
                            power_password_confirm: {equalTo: '#power_password'}
                        }
                    });
                }
                else if($form.hasClass('fwupgrade-password')){
                    $form.validate({
                        submitHandler : submitPasswordHandler,
                        rules: {
                            fwupgrade_password_confirm: {equalTo: '#fwupgrade_password'}
                        }
                    });
                }
                else{
                    $form.validate({
                        submitHandler : submitPasswordHandler
                    });
                }
            });
        },

        initRecoveryPassword = function($context){
            if($context === undefined){ $context = $("#main"); }
            var $forms = $context.find("form");

            $.each($forms, function(i,el){
                var $form = $(el);
                $form.submit(function(e){e.preventDefault();});
                $form.validate({
                    submitHandler : submitPasswordRecoveryCode
                });
            });
        },

    //generic submitHandler that sends the form after pushing the token in
        submitHandler = function submitHandler(form){

            var $form = $(form),
                url = $form.attr("action"),
                method = $form.attr("method"),
                data = $form.serializeArray({ checkboxesAsBools: true });

            data = _.chain(data).indexBy('name').mapValues('value').value();

            data.token = Cookies.get("token");

            var xhr = $.ajax({
                url: url,
                method: method,
                data: data
            });
            xhr.done(function(data){
                var res = JSON.parse(data);
                if(res.response === "ok"){
                    showSuccess(res.message);
                }else{
                    showError(res.message);
                }
            });
            xhr.error(function(err){
                showError(err.statusText);
            });

            return xhr;
        },

        submitPasswordHandler = function submitPasswordHandler(form){

            var $form = $(form),
                $submit = $form.find("input[type=submit]"),
                url = $form.attr("action"),
                method = $form.attr("method"),
                data = $form.serializeArray({ checkboxesAsBools: true }),
                submitTxt = $submit.val(),
                pwd;

            data = _.chain(data).indexBy('name').mapValues('value').value();

            //append token to sent data
            data.token = Cookies.get("token");

            if($form.hasClass("admin-password")){
                pwd = data.admin_password;
                if(pwd !== "" && pwd === data.admin_password_confirm){
                    data.password = pwd;
                }
                delete data.admin_password;
                delete data.admin_password_confirm;
            }
            else if($form.hasClass("power-password")){
                pwd = data.power_password;
                if(pwd !== "" && pwd === data.power_password_confirm){
                    data.password = pwd;
                }
                delete data.power_password;
                delete data.power_password_confirm;
            }
            else if($form.hasClass("fwupgrade-password")){
                pwd = data.fwupgrade_password;
                if(pwd !== "" && pwd === data.fwupgrade_password_confirm){
                    data.password = pwd;
                }
                delete data.fwupgrade_password;
                delete data.fwupgrade_password_confirm;
            }else{
                // a generic "password"
                pwd = data.password;
                if(pwd !== "" && pwd === data.password_confirm){
                    data.password = pwd;
                }
                delete data.password_confirm;
            }

            //btn to saving state
            $submit.val(Config.get("waitMessage")).prop("disabled", "disabled");

            var xhr = $.ajax({
                url: url,
                method: method,
                data: data
            });

            xhr.done(function(data){
                var res = JSON.parse(data);

                if(res.response === "ok"){
                    showSuccess(res.message);
                    $submit.val(Config.get("btnSaveSuccessMessage"))
                        .removeClass("btn-primary")
                        .addClass("btn-success");
                }else{
                    showError(res.message);
                    $submit.val(Config.get("btnSaveErrorMessage"))
                        .removeClass("btn-primary")
                        .addClass("btn-danger");
                }
            });

            xhr.error(function(err){
                showError(err.statusText);
                $submit.val(Config.get("btnSaveErrorMessage"))
                    .removeClass("btn-primary")
                    .addClass("btn-danger");
            });

            xhr.always(function(){
                //btn back to initial state
                window.setTimeout(function(){
                    $submit.val(submitTxt)
                        .prop("disabled", false)
                        .removeClass("btn-danger")
                        .removeClass("btn-success")
                        .addClass("btn-primary");
                }, Config.get("feedbackMessageDuration"));
            });
        },

        submitPasswordRecoveryCode = function submitPasswordRecoveryCode(form){

            var $form = $(form),
                $submit = $form.find("input[type=submit]"),
                url = $form.attr("action"),
                method = $form.attr("method"),
                data = $form.serializeArray({ checkboxesAsBools: true }),
                submitTxt = $submit.val();

            data = _.chain(data).indexBy('name').mapValues('value').value();

            //append token to sent data
            data.token = Cookies.get("token");

            //btn to saving state
            $submit.val(Config.get("waitMessage")).prop("disabled", "disabled");

            var xhr = $.ajax({
                url: url,
                method: method,
                data: data
            });

            xhr.done(function(data){
                var res = JSON.parse(data);

                if(res.response === "ok"){
                    showSuccess(res.message);
                    $submit.val(Config.get("btnSaveSuccessMessage"))
                        .removeClass("btn-primary")
                        .addClass("btn-success");
                }else{
                    showError(res.message);
                    $submit.val(Config.get("btnSaveErrorMessage"))
                        .removeClass("btn-primary")
                        .addClass("btn-danger");
                }
            });

            xhr.error(function(err){
                showError(err.statusText);
                $submit.val(Config.get("btnSaveErrorMessage"))
                    .removeClass("btn-primary")
                    .addClass("btn-danger");
            });

            xhr.always(function(){
                //btn back to initial state
                window.setTimeout(function(){
                    $submit.val(submitTxt)
                        .prop("disabled", false)
                        .removeClass("btn-danger")
                        .removeClass("btn-success")
                        .addClass("btn-primary");
                }, Config.get("feedbackMessageDuration"));
            });
        },

        submitModalHandler = function submitModalHandler(form){

            var $form = $(form),
                url = $form.attr("action"),
                method = $form.attr("method"),
                data = $form.serializeArray({ checkboxesAsBools: true }),
                $modal = $form.closest(".modal");

            data.token = Cookies.get("token");

            var xhr = $.ajax({
                url: url,
                method: method,
                data: data
            });
            xhr.done(function(data){
                var res = JSON.parse(data);
                if(res.response === "ok"){
                    showSuccess(res.message);
                    if($form.hasClass("js-reboot")){
                        rebootModal();
                        //avoid spamming useless ajax calls while rebooting
                        clearIntervals();
                    }
                }else{
                    showError(res.message);
                }
            });
            xhr.error(function(err){
                showError(err.statusText);
            });
            xhr.always(function(){
                $modal.modal('hide');
            });
        },

        removeLoader = function removeLoader(){
            vars.$loader.fadeOut(400);
            if(vars.liveData.requires_reboot === 2){
                showRebootInfo(vars.popupReboot, true);
            }
            else if(vars.liveData.requires_reboot === 1){
                showRebootInfo(vars.popupReload, true);
            }
        },

        selectCurrentPage = function selectCurrentPage(){
            // adds the 'active' class in the main header,for highlighting
            // the right 'current page'
            vars.$header.find("#nav ul.nav li."+vars.$body.data("page")).addClass("active");
        },

        checkReboot = function checkReboot(){
            if(vars.liveData.requires_reboot === 2){
                var rebootRq = Cookies.get("rebootRequest");
                if(rebootRq !== undefined && rebootRq.indexOf("RELOAD NOW") > 0) {
                    Cookies.remove("rebootRequest");
                    $.notifyClose("top-right");
                }
                else
                    showRebootInfo(vars.popupReboot, false);
            }
            else if(vars.liveData.requires_reboot === 1){
                showRebootInfo(vars.popupReload, false);
            }
            else {
                var rebootRq = Cookies.get("rebootRequest");
                if(rebootRq !== undefined){
                    Cookies.remove("rebootRequest");
                    $.notifyClose("top-right");
                }
            }
        },

        rebootModal = function rebootModal(sec){
            if(sec === undefined){ sec = 180; }

            var $rebootingModal = $('#rebooting-modal');

            $rebootingModal.modal({backdrop:"static", keyboard:false});

            vars.rebootIn = {total: sec, remaining: sec};

            $rebootingModal.find(".seconds").html(vars.rebootIn.remaining);

            var rebootTimer = setInterval(function(){
                if(vars.rebootIn.remaining<=0){
                    //close modal, clear the setInterval and reset the modal.
                    $rebootingModal.modal('hide');
                    clearInterval(rebootTimer);
                    $rebootingModal.find(".progress-bar").width("0%");
                    window.location.href = 'login.html';
                }else{
                    vars.rebootIn.remaining--;

                    $rebootingModal.find(".seconds").html(vars.rebootIn.remaining);
                    var perc = calculatePercentage(vars.rebootIn.remaining, vars.rebootIn.total);
                    $rebootingModal.find(".progress-bar").width(perc+"%");
                }
            }, 1000);
        },

        setLocale = function setLocale(locale){
            $.datetimepicker.setLocale(locale);
        },

    // INIT

        initSensorsData = function initSensorsData(){

            var $widgetWrapper = $(".js-sensors-widgets"),
                widgetTpls = {
                    'io' : [
                        "<div class='col-xs-6'>",
                        "  <div class='widget-box widget-box--sensor'>",
                        "    <h3 class='sub-heading'>Temperature & Digital I/O</h3>",
                        "    <strong class='value-info'>",
                        "      <div class='row comm-lost'>",
                        "        <div class='col-xs-12'>",
                        "          <small>Communication Lost</small>",
                        "        </div> ",
                        "      </div> ",
                        "      <div class='row'>",
                        "        <div class='col-xs-4'>",
                        "          <small class='icoChar temperature'>: <span class='temp'></span>°C</small>",
                        "        </div>",
                        "        <div class='col-xs-4'>",
                        "          <small class='icoChar input'>: <span class='in'></span></small>",
                        "        </div>",
                        "        <div class='col-xs-4'>",
                        "          <small class='icoChar output'>: <span class='out'></span></small>",
                        "        </div>",
                        "      </div>",
                        "    </strong>",
                        "  </div>",
                        "</div>"
                    ].join(" "),

                    'humidity' : [
                        "<div class='col-xs-4'>",
                        "  <div class='widget-box widget-box--sensor'>",
                        "    <h3 class='sub-heading'>Temperature & Humidity</h3>",
                        "    <strong class='value-info'>",
                        "      <div class='row comm-lost'>",
                        "        <div class='col-xs-12'>",
                        "          <small>Communication Lost</small>",
                        "        </div> ",
                        "      </div> ",
                        "      <div class='row'>",
                        "        <div class='col-xs-5'>",
                        "          <small class='icoChar temperature'>: <span class='temp'></span>°C</small>",
                        "        </div>",
                        "        <div class='col-xs-7'>",
                        "          <small class='icoChar humidity'>: <span></span>%RH</small>",
                        "        </div>",
                        "      </div>",
                        "    </strong>",
                        "  </div>",
                        "</div>"
                    ].join(" "),

                    'temperature': [
                        "<div class='col-xs-2'>",
                        "  <div class='widget-box widget-box--sensor'>",
                        "    <h3 class='sub-heading'>Temperature</h3>",
                        "    <strong class='value-info'>",
                        "      <div class='row comm-lost'>",
                        "        <div class='col-xs-12'>",
                        "          <small>Comm. Lost</small>",
                        "        </div> ",
                        "      </div> ",
                        "      <div class='row'>",
                        "        <div class='col-xs-12'>",
                        "          <small class='icoChar temperature'>: <span class='temp'></span>°C</small>",
                        "        </div>",
                        "      </div>",
                        "    </strong>",
                        "  </div>",
                        "</div>"
                    ].join(" "),

                    'unknown': [
                        "<div class='col-xs-4'>",
                        "  <div class='widget-box widget-box--sensor'>",
                        "    <h3 class='sub-heading'>ERROR: unknown sensor</h3>",
                        "    <strong class='sub-title'>No data available for this sensor.</strong>",
                        "  </div>",
                        "</div>"
                    ].join(" "),

                    'no-sensor': [
                        "<div class='col-xs-12'>",
                        "  No sensor present.",
                        "</div>"
                    ].join(" ")
                },
                i,
                tpl,
                sensors = vars.liveData.sensors;

            $widgetWrapper.empty();

            if(sensors && sensors.length > 0){
                for(i=0; i<sensors.length; i++){

                    // TODO: IF comm IS NOT undefined or IF it IS true ?

                    /*  da vars.liveData.sensors
                     "comm" : true, sensore1 comunicazione  (true = attiva)
                     "temp" : 362 , sensore1 temperatura ( / 10)
                     "in"   : true, sensore1 ingresso (true = aperto, false = chiuso)
                     "out"  : true, sensore1 uscita (true = aperto, false = chiuso)

                     "comm" : true, sensore2 comunicazione  (true = attiva)
                     "temp" : 337 , sensore2 temperatura ( / 10)
                     "hum"  : 18    sensore2 umidità in %
                     */
                    if(sensors[i]['comm'] !== undefined){

                        var sensorType;

                        if(sensors[i]['in'] !== undefined && sensors[i]['in'] !== undefined){
                            sensorType = 'io';
                        }else if(sensors[i]['hum'] !== undefined){
                            sensorType = 'humidity';
                        }else if(sensors[i]['temp'] !== undefined){
                            sensorType = 'temperature';
                        }
                        else{
                            sensorType = 'unknown';
                        }

                        tpl = $(widgetTpls[sensorType]).clone();

                        if(sensors[i].comm === false){
                            tpl.find(".comm-lost").removeClass("hidden").siblings().addClass("hidden");
                        }else{
                            tpl.find(".comm-lost").addClass("hidden");
                        }

                        switch(sensorType){
                            case 'io':
                                tpl.find('span.in').text( sensors[i]['in'] ? Config.get("ioSensorsStatus").in[1] : Config.get("ioSensorsStatus").in[0] );
                                tpl.find('span.out').text( sensors[i]['out'] ? Config.get("ioSensorsStatus").out[1] : Config.get("ioSensorsStatus").out[0] );
                                tpl.find('span.temp').text( sensors[i]['temp'] / 10 );
                                break;
                            case 'humidity':
                                tpl.find('span').text( sensors[i]['hum'] );
                                tpl.find('span.temp').text( sensors[i]['temp'] / 10 );

                                break;
                            case 'temperature':
                                tpl.find('span.temp').text( sensors[i]['temp'] / 10 );
                                break;
                            default:
                                break;
                        }

                        tpl.appendTo($widgetWrapper);
                    }
                }
            }else{
                tpl = $(widgetTpls['no-sensor']).clone();
                tpl.appendTo($widgetWrapper);
            }
        },

        initLogin = function initLogin(){

            var $form =$(".js-login-form");

            $("#logintype").on("change", function (e) {
                if (document.getElementById("logintype").value.includes("ldap")) {
                    // se LDAP blocco user name
                    //$('#user').hide();
                    //$('#user').prop("disabled", true);
                    $('#user').val("");
                    $('#password').val("");
                //$('#user').val(document.getElementById("logintype").value);
                }
                else{
                    //$('#user').prop("disabled", false);
                    $('#user').val("");
                    $('#password').val("");
                }
            })

            $form.submit(function(e){e.preventDefault();});

            $form.validate({submitHandler : function(form){

                var $form = $(form),
                    $submit = $form.find(".js-login-btn"),
                    url = $form.attr("action"),
                    method = $form.attr("method"),
                    data = $form.serializeArray({ checkboxesAsBools: true }),
                    submitDefaultText = $submit.val();

                data = _.chain(data).indexBy('name').mapValues('value').value();

                //btn to saving state
                $submit.prop("disabled", "disabled").val(Config.get("waitMessage"));

                var xhr = $.ajax({
                    url: url,
                    method: method,
                    data: data
                });

                xhr.done(function(data){
                    var res = JSON.parse(data);

                    if(res.response === "ok"){
                        setSuccess(Config.get("loginSuccessMessage"));
                        Cookies.set('token', res.data.token);
                        window.location.href = "index.html";
                    }else{
                        showError(res.message);
                    }
                });

                xhr.error(function(err){
                    showError(err.statusText);
                });

                xhr.always(function(){
                    $submit.prop("disabled", false).val(submitDefaultText);
                });
            }
            });
        },

        initCustomForms = function initCustomForms() {
            // xdSofr DatePicker Initialization
            var $datetimePicker = $(".datetimepicker");
            var $datePicker = $(".datepicker");

            $datePicker.datetimepicker({timepicker: false, format:'d/m/Y'});
            $datetimePicker.datetimepicker({format:'d/m/Y H:i'});

            // DateTimePicker icon button
            $datetimePicker
                .closest(".date-input")
                .find(".btn-pick")
                .on("click", function(e){
                    e.preventDefault();
                    $(this).siblings(".date-pick").first().find(".datetimepicker").datetimepicker('show')
                });

            // DatePicker icon button
            $datePicker
                .closest(".date-input")
                .find(".btn-pick")
                .on("click", function(e){
                    e.preventDefault();
                    $(this).siblings(".date-pick").first().find(".datepicker").datetimepicker('show')
                });


            // jcf JavascriptCustomForms init
            jcf.setOptions('Select', { wrapNative: false });
            jcf.replaceAll();

            $(".dropzone").each(function(i, e){
                var $this = $(e);
                $this.dropzone({
                    url:nullReplace($this.attr("action"), $this.data("url")),
                    maxFiles : $this.data('multiple') === false ? 1 : null,
                    dictDefaultMessage: nullReplace($this.data("message"), 'Drop files here to upload'),
                    acceptedFiles: nullReplace($this.data("filetypes"), ''),
                    forceFallback : false,
                    success : function(file, res){
                        res = $.parseJSON(res);
                        if(res.response === "error"){
                            var node, _i, _len, _ref, _results;
                            var message = res.message;
                            file.previewElement.classList.add("dz-error");
                            _ref = file.previewElement.querySelectorAll("[data-dz-errormessage]");
                            _results = [];
                            for (_i = 0, _len = _ref.length; _i < _len; _i++) {
                                node = _ref[_i];
                                _results.push(node.textContent = message);
                            }
                            return _results;
                        }else{
                            if($(this.element).hasClass("js-firmware-upload")){
                                Main.reboot();
                            }
                            return file.previewElement.classList.add("dz-success");
                        }
                    },
                    previewTemplate : [
                        '<div class="dz-preview dz-file-preview">',
                        '  <div class="dz-image"><img data-dz-thumbnail /></div>',
                        '  <div class="dz-details">',
                        '    <div class="dz-size"><span data-dz-size></span></div>',
                        '    <div class="dz-filename"><span data-dz-name></span></div>',
                        '  </div>',
                        '  <div class="dz-progress"><span class="dz-upload" data-dz-uploadprogress></span></div>',
                        '  <div class="dz-error-message"><span data-dz-errormessage></span></div>',
                        '  <div class="dz-success-mark">',
                        '    <svg width="54px" height="54px" viewBox="0 0 54 54" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink">',
                        '      <g stroke="none" stroke-width="1" fill="none" fill-rule="evenodd">',
                        '        <path d="M23.5,31.8431458 L17.5852419,25.9283877 C16.0248253,24.3679711 13.4910294,24.366835 11.9289322,25.9289322 C10.3700136,27.4878508 10.3665912,30.0234455 11.9283877,31.5852419 L20.4147581,40.0716123 C20.5133999,40.1702541 20.6159315,40.2626649 20.7218615,40.3488435 C22.2835669,41.8725651 24.794234,41.8626202 26.3461564,40.3106978 L43.3106978,23.3461564 C44.8771021,21.7797521 44.8758057,19.2483887 43.3137085,17.6862915 C41.7547899,16.1273729 39.2176035,16.1255422 37.6538436,17.6893022 L23.5,31.8431458 Z M27,53 C41.3594035,53 53,41.3594035 53,27 C53,12.6405965 41.3594035,1 27,1 C12.6405965,1 1,12.6405965 1,27 C1,41.3594035 12.6405965,53 27,53 Z" id="Oval-2" stroke-opacity="0.198794158" stroke="#747474" fill-opacity="0.816519475" fill="#FFFFFF" sketch:type="MSShapeGroup"></path>',
                        '      </g>',
                        '    </svg>',
                        '  </div>',
                        '  <div class="dz-error-mark">',
                        '    <svg width="54px" height="54px" viewBox="0 0 54 54" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink">',
                        '      <g stroke="none" stroke-width="1" fill="none" fill-rule="evenodd">',
                        '        <g stroke="#747474" stroke-opacity="0.198794158" fill="#FFFFFF" fill-opacity="0.816519475">',
                        '          <path d="M32.6568542,29 L38.3106978,23.3461564 C39.8771021,21.7797521 39.8758057,19.2483887 38.3137085,17.6862915 C36.7547899,16.1273729 34.2176035,16.1255422 32.6538436,17.6893022 L27,23.3431458 L21.3461564,17.6893022 C19.7823965,16.1255422 17.2452101,16.1273729 15.6862915,17.6862915 C14.1241943,19.2483887 14.1228979,21.7797521 15.6893022,23.3461564 L21.3431458,29 L15.6893022,34.6538436 C14.1228979,36.2202479 14.1241943,38.7516113 15.6862915,40.3137085 C17.2452101,41.8726271 19.7823965,41.8744578 21.3461564,40.3106978 L27,34.6568542 L32.6538436,40.3106978 C34.2176035,41.8744578 36.7547899,41.8726271 38.3137085,40.3137085 C39.8758057,38.7516113 39.8771021,36.2202479 38.3106978,34.6538436 L32.6568542,29 Z M27,53 C41.3594035,53 53,41.3594035 53,27 C53,12.6405965 41.3594035,1 27,1 C12.6405965,1 1,12.6405965 1,27 C1,41.3594035 12.6405965,53 27,53 Z" id="Oval-2" sketch:type="MSShapeGroup"></path>',
                        '        </g>',
                        '      </g>',
                        '    </svg>',
                        '  </div>',
                        '</div>'].join(""),
                    init: function(){
                        this.on("maxfilesexceeded", function(file){
                            this.removeFile(file);
                        });
                        this.on("error", function(file, err){
                            showError(err.message);
                        });

                        this.on('thumbnail', function(file, dataUrl) {
                            // Display the image in your file.previewElement
                        });
                        /*
                         if($(this.element).hasClass("js-firmware-upload")){
                         this.on("success", function(file){
                         showSuccess("Success Text Here");
                         rebootModal(125);
                         })
                         }else{
                         this.on("success", function(file){
                         showSuccess("Success Text Here");
                         })
                         }
                         */
                    }
                });

            });


            // jQuery Validate options (overrides jquery validate plugin defaults)
            $.validator.setDefaults({
                highlight: function(element) {
                    $(element).closest('.form-group').addClass('has-error');
                },
                unhighlight: function(element) {
                    $(element).closest('.form-group').removeClass('has-error');
                },
                errorElement: 'span',
                errorClass: 'help-block',
                errorPlacement: function(error, element) {
                    if(element.parent('.input-group').length) {
                        error.insertAfter(element.parent());
                    } else {
                        error.insertAfter(element);
                    }
                },
                ignore:''
            });

            // Riferimento
            // http://stackoverflow.com/questions/106179/regular-expression-to-match-dns-hostname-or-ip-address
            $.validator.addMethod("hostname", function(value, element) {
                return this.optional(element) || /^[a-z0-9\-]+$/i.test(value);
            }, "Insert a valid hostname");

            $.validator.addMethod("macaddress", function(value, element) {
                return this.optional(element) || /^([0-9A-Fa-f]{2}[:-]){5}([0-9A-Fa-f]{2})$/i.test(value);
            }, "Insert a valid MAC Address");

            // ip
            $.validator.addMethod("fullHost", function(value, element) {
                return this.optional(element) || /^[a-z0-9\-.]+$/i.test(value) || /^(25[0-5]|2[0-4]\d|[01]?\d\d?)\.(25[0-5]|2[0-4]\d|[01]?\d\d?)\.(25[0-5]|2[0-4]\d|[01]?\d\d?)\.(25[0-5]|2[0-4]\d|[01]?\d\d?)$/i.test(value);
            }, "Insert a valid hostname or a valid IPv4");

            // lowercase alphanumeric
            $.validator.addMethod("safecharacters", function (value, element) {
                return this.optional(element) || /^[a-z0-9,._+:@%/-]+$/i.test(value);
            }, "Special characters allowed are ,._+:@%/-");

            // simple phone number
            $.validator.addMethod("simplephone", function (value, element) {
                return this.optional(element) || /^([0-9+]{4,19})+$/i.test(value);
            }, "Insert a valid phone number");
        },

        initEventsHandler = function initEventsHandler() {
            var $rebootModal = $("#reboot-modal"),
                $restartModal = $("#restart-modal"),
                $resetDefaultsModal = $("#reset-defaults-modal"),
                $onbypassModal = $("#confirm-onbypass-modal"),
                $shutdownModal = $("#confirm-shutdown-modal"),
                $shutdownXTSModal = $("#confirm-shutdown-xts-modal"),
                $shutdownMDUModal = $("#confirm-shutdown-mdu-modal"),
                $shutdownIRMSModal = $("#confirm-shutdown-irms-modal"),
                $rebootXTSModal = $("#confirm-reboot-xts-modal"),
                $rebootMDUModal = $("#confirm-reboot-mdu-modal"),
                $rebootIRMSModal = $("#confirm-reboot-irms-modal"),
                $showmodalshutdown = $("#show-modal-shutdown"),
                $shutdowndelay = $("#shutdown-delay"),
                $shutdownxtsdelay = $("#shutdown-xts-delay"),
                $shutdownmdudelay = $("#shutdown-mdu-delay"),
                $mduoutputsocket = $("#shutdown-mdu-output-socket"),
                $irmsoutputsocket = $("#shutdown-irms-output-socket"),
                $rebootxtsdelay = $("#reboot-xts-delay"),
                $rebootmdudelay = $("#reboot-mdu-delay"),
                $shutdownRebootModal = $("#confirm-shutdown-restore-modal"),
                $shutdownRebootXTSModal = $("#confirm-restore-xts-modal"),
                $shutdownRebootMDUModal = $("#confirm-restore-mdu-modal"),
                $shutdownRebootIRMSModal = $("#confirm-restore-irms-modal"),
                $showmodalshutdownReboot = $("#show-modal-shutdown-reboot"),
                $shutdownrebootDelay = $("#shutdown-restore-delay"),
                $shutdownrebootXTSDelay = $("#restore-xts-delay"),
                $shutdownrebootMDUDelay = $("#restore-mdu-delay"),
                $restoredelay = $("#restore-delay"),
                $showmodalshutdownxts = $("#show-modal-shutdown-xts"),
                $showmodalshutdownmdu = $("#show-modal-shutdown-mdu"),
                $showmodalshutdownirms = $("#show-modal-shutdown-irms"),
                $showmodalshutdownrestorexts = $("#show-modal-shutdown-restore-xts"),
                $showmodalshutdownrestoremdu = $("#show-modal-shutdown-restore-mdu"),
                $showmodalshutdownrestoreirms = $("#show-modal-shutdown-restore-irms"),
                $restoremduoutputsocket = $("#restore-mdu-output-socket"),
                $restoreirmsoutputsocket = $("#restore-irms-output-socket"),
                $rebootmduoutputsocket = $("#reboot-mdu-output-socket"),
                $rebootirmsoutputsocket = $("#reboot-irms-output-socket"),
                $showmodalrebootxts = $("#show-modal-reboot-xts"),
                $showmodalrebootmdu = $("#show-modal-reboot-mdu"),
                $showmodalrebootirms = $("#show-modal-reboot-irms"),
                $switchIRMSModal = $("#confirm-switch-irms-modal"),
                $showmodalswitchirms = $("#show-modal-switch-irms"),
                $switchirmsoutputsocket = $("#switch-irms-output-socket"),
                $switchirmsinputsource = $("#switch-irms-input-source"),
                $resetLogModal = $("#reset-log-modal");

            vars.randomString = randomString(10);

            // ------- onbypass modal -----

            var $onbypassForm = $onbypassModal.find("form");
            $onbypassForm.submit(function(e){e.preventDefault();});
            $onbypassForm.validate({
                submitHandler : submitModalHandler
            });

            // ------- shutdown modal -----

            var $shutdownForm = $shutdownModal.find("form");
            $showmodalshutdown.on("click", function () {
                $shutdownModal.find("#shutdown-modal-seconds-txt").html($shutdowndelay.val());
                $shutdownForm.find("#shutdown-delay").val($shutdowndelay.val());
            });
            $shutdownForm.validate({
                submitHandler : submitModalHandler
            });

            // ------- shutdown XTS modal -----

            var $shutdownXTSForm = $shutdownXTSModal.find("form");
            $showmodalshutdownxts.on("click", function () {
                $shutdownXTSModal.find("#shutdown-modal-seconds-txt").html($shutdownxtsdelay.val());
                $shutdownXTSForm.find("#shutdown-delay").val($shutdownxtsdelay.val());
            });
            $shutdownXTSForm.validate({
                submitHandler : submitModalHandler
            });

            // ------- shutdown MDU modal -----

            var $shutdownMDUForm = $shutdownMDUModal.find("form");
            $showmodalshutdownmdu.on("click", function () {
               	$shutdownMDUModal.find("#shutdown-modal-seconds-txt").html($shutdownmdudelay.val());
		if ($mduoutputsocket.val() === ("A")){
	                $shutdownMDUModal.find("#socket-num").html("all the sockets");
		}
		else {
	                $shutdownMDUModal.find("#socket-num").html("the socket "+$mduoutputsocket.val());
		}
                $shutdownMDUForm.find("#shutdown-delay-sd-mdu").val($shutdownmdudelay.val());
                $shutdownMDUForm.find("#output-socket-sd-mdu").val($mduoutputsocket.val());
            });
            $shutdownMDUForm.validate({
                submitHandler : submitModalHandler
            });

            // ------- shutdown IRMS modal -----

            var $shutdownIRMSForm = $shutdownIRMSModal.find("form");
            $showmodalshutdownirms.on("click", function () {
                $shutdownIRMSModal.find("#socket-num-sd-irms").html($irmsoutputsocket.val());
                $shutdownIRMSForm.find("#output-socket-sd-irms").val($irmsoutputsocket.val());
            });
            $shutdownIRMSForm.validate({
                submitHandler : submitModalHandler
            });

            // ------- reboot XTS modal -----

            var $rebootXTSForm = $rebootXTSModal.find("form");
            $showmodalrebootxts.on("click", function () {
                $rebootXTSModal.find("#minutes-txt").html($rebootxtsdelay.val());
                $rebootXTSForm.find("#reboot-delay").val($rebootxtsdelay.val());
            });
            $rebootXTSForm.validate({
                submitHandler : submitModalHandler
            });

            // ------- reboot MDU modal -----

            var $rebootMDUForm = $rebootMDUModal.find("form");
            $showmodalrebootmdu.on("click", function () {
                $rebootMDUModal.find("#minutes-txt").html($rebootmdudelay.val());
		if ($rebootmduoutputsocket.val() === ("A")){
	                $rebootMDUModal.find("#socket-num").html("all the sockets");
		}
		else {
                	$rebootMDUModal.find("#socket-num").html("the socket "+$rebootmduoutputsocket.val());
		}
                $rebootMDUForm.find("#reboot-delay").val($rebootmdudelay.val());
                $rebootMDUForm.find("#output-socket").val($rebootmduoutputsocket.val());
            });
            $rebootMDUForm.validate({
                submitHandler : submitModalHandler
            });

            // ------- reboot IRMS modal -----

            var $rebootIRMSForm = $rebootIRMSModal.find("form");
            $showmodalrebootirms.on("click", function () {
                $rebootIRMSModal.find("#socket-num").html($rebootirmsoutputsocket.val());
                $rebootIRMSForm.find("#output-socket-rb-irms").val($rebootirmsoutputsocket.val());
            });
            $rebootIRMSForm.validate({
                submitHandler : submitModalHandler
            });

            // ------- shutdown-restart modal -----

            var $shutdownRebootForm = $shutdownRebootModal.find("form");
            $showmodalshutdownReboot.on("click", function () {
                $shutdownRebootModal.find("#seconds-txt").html($shutdownrebootDelay.val());
                $shutdownRebootModal.find("#minutes-txt").html($restoredelay.val());
                $shutdownRebootForm.find("#shutdown-delay-sr-ups").val($shutdownrebootDelay.val());
                $shutdownRebootForm.find("#restore-delay-sr-ups").val($restoredelay.val());
            });
            $shutdownRebootForm.validate({
                submitHandler : submitModalHandler
            });
            // ------- shutdown-restart XTS modal -----

            var $shutdownRebootXTSForm = $shutdownRebootXTSModal.find("form");
            $showmodalshutdownrestorexts.on("click", function () {
                $shutdownRebootXTSModal.find("#minutes-txt").html($shutdownrebootXTSDelay.val());
                $shutdownRebootXTSForm.find("#restore-delay").val($shutdownrebootXTSDelay.val());
            });
            $shutdownRebootXTSForm.validate({
                submitHandler : submitModalHandler
            });
            // ------- shutdown-restart MDU modal -----

            var $shutdownRebootMDUForm = $shutdownRebootMDUModal.find("form");
            $showmodalshutdownrestoremdu.on("click", function () {
                $shutdownRebootMDUModal.find("#minutes-txt").html($shutdownrebootMDUDelay.val());
		if ($restoremduoutputsocket.val() === ("A")){
	                $shutdownRebootMDUModal.find("#socket-num").html("all the sockets");
		}
		else {
	                $shutdownRebootMDUModal.find("#socket-num").html("the socket "+$restoremduoutputsocket.val());
		}
                $shutdownRebootMDUForm.find("#restore-delay-sr-mdu").val($shutdownrebootMDUDelay.val());
                $shutdownRebootMDUForm.find("#output-socket-sr-mdu").val($restoremduoutputsocket.val());
            });
            $shutdownRebootMDUForm.validate({
                submitHandler : submitModalHandler
            });
            // ------- shutdown-restart IRMS modal -----

            var $shutdownRebootIRMSForm = $shutdownRebootIRMSModal.find("form");
            $showmodalshutdownrestoreirms.on("click", function () {
                $shutdownRebootIRMSModal.find("#socket-num").html($restoreirmsoutputsocket.val());
                $shutdownRebootIRMSForm.find("#output-socket-rs-irms").val($restoreirmsoutputsocket.val());
            });
            $shutdownRebootIRMSForm.validate({
                submitHandler : submitModalHandler
            });
            // ------- switch-input IRMS modal -----

            var $switchinputIRMSForm = $switchIRMSModal.find("form");
            $showmodalswitchirms.on("click", function () {
                $switchIRMSModal.find("#socket-num").html($switchirmsoutputsocket.val());
                $switchIRMSModal.find("#input-name").html($switchirmsinputsource.val());
                $switchinputIRMSForm.find("#output-socket").val($switchirmsoutputsocket.val());
                $switchinputIRMSForm.find("#input-source").val($switchirmsinputsource.val());
            });
            $switchinputIRMSForm.validate({
                submitHandler : submitModalHandler
            });


            // ------- reboot modal -------
            var $rebootForm = $rebootModal.find("form");
            $rebootForm.submit(function(e){e.preventDefault();});

            $rebootForm.validate({
                submitHandler : submitModalHandler
            });

            // ------- restart modal -------
            var $restartForm = $restartModal.find("form");
            $restartForm.submit(function(e){
				e.preventDefault();
                vars.lastSaveConfigButton = moment(new Date()).format("X");
				});

            $restartForm.validate({
                submitHandler : submitModalHandler
            });

            // ------- reset to defaults modal -------

            var $resetDefaultsForm = $resetDefaultsModal.find('form');
            $resetDefaultsForm.submit(function(e){e.preventDefault();});

            // init random string for resetDefaults
            $resetDefaultsForm.find("#reset-default-captcha").html(vars.randomString);
            $resetDefaultsForm.find("#reset_captcha").val(vars.randomString);

            $resetDefaultsForm.validate({
                rules: {
                    'reset_captcha_confirm': {equalTo: '#reset_captcha'}
                },
                submitHandler : submitModalHandler
            });

            // ------- reset log modal -------
            var $resetLogForm = $resetLogModal.find('form');
            $resetLogForm.submit(function(e){e.preventDefault();});

            $resetLogForm.validate({
                rules: {
                    'reset_captcha_confirm': {equalTo: '#reset_captcha'}
                },
                submitHandler : submitModalHandler
            });

            // ------- test battery -------
            var $testBatteryBtn = $(".js-test-battery");
            var $testBatteryModal = $("#test-battery-modal");

            $testBatteryBtn.on("click", function(e) {
                e.preventDefault();

                $("#battery-testing").show();
                $("#battery-test-result").hide();

                $testBatteryModal.modal("show");

                var xhr = $.ajax({
                    url: 'cgi-bin/command_test_battery.cgi',
                    method: 'post',
                    data: { 'token': Cookies.get("token") },
                    dataType: 'json'
                });
                xhr.done(function(res){
                    console.log(res);
                    if(res.response === "ok"){
                        $("#battery-testing").hide();
                        $("#battery-test-result").show();
                    }else{
                        showError(res.message);
                        $testBatteryModal.modal('hide');
                    }
                });
                xhr.error(function(err){
                    showError(err.statusText);
                    $testBatteryModal.modal('hide');
                });
            });


        },

        initNotifications = function initNotifications(){
            var ntf = Cookies.get("notifications");
            if(ntf !== undefined){
                //load notifications
                ntf = JSON.parse(ntf);

                //print them
                $.each(ntf, function(i, n){
                    showMessage(n.message, n.type, true);
                });

                //show only once
                Cookies.remove("notifications");
            }
        },

        initDownloadButton = function initDownloadButton(){
            $(".js-download-service-log").on("click", function(){
                window.location.href = Config.get("serviceLogDownloadSource");
            });
        },

        initTabNavigation = function initTabNavigation(tabContainer){
            // Javascript to enable direct linking to tabs.
            //
            // if linking a page like "mypage.html#my_tab" the page will be
            // loaded with "my_tab" selected AND the page will automatically
            // scroll to the anchor.
            //
            // If you want to open the tab without scrolling to the anchor,
            // link the tab with a 'tab_' prefix, like:
            // mypage.html#tab_my_tab

            var hash = document.location.hash,
                prefix = "tab_",
                $tabContainer = $(tabContainer);

            if (hash) {
                $tabContainer.find('a[href="'+hash.replace(prefix,"")+'"]').tab('show');
            }

            // Change hash for page-reload
            $tabContainer.find('a').on('shown.bs.tab', function (e) {
                window.location.hash = e.target.hash.replace("#", "#" + prefix);
            });
        },

        initHeader = function initHeader(){
            selectCurrentPage();

            $('.user-drop-holder').touchHover();
            $('.alarm-drop-holder').touchHover();

            $("#header").find(".logout").click(function(e){ e.preventDefault();
                doLogout();
            });

//            $("#header").find(".btn-test").on("click", function(){
//                showSuccess(Config.get("loginSuccessMessage"));
//            });

        },

        isMonoMono = function() {
            return (vars.nominalData.io_conf == 1);
        },

        isTriMono = function() {
            return (vars.nominalData.io_conf == 3);
        },

        isTriTri = function() {
            return (vars.nominalData.io_conf == 4);
        },

        isATS = function() {
            return (vars.nominalData.ups_type == 5);
        },

        setLastSaveConfigButton = function() {
            vars.lastSaveConfigButton = moment(new Date()).format("X");
        },
    // ----- Init -----

        initCustomValidators = function () {

            /**
             * Dutch phone numbers have 10 digits (or 11 and start with +31).
             */
            $.validator.addMethod( "phoneIT", function( value, element ) {
                return this.optional( element ) || /^((\+|00(\s|\s?\-\s?)?)31(\s|\s?\-\s?)?(\(0\)[\-\s]?)?|0)[1-9]((\s|\s?\-\s?)?[0-9]){8}$/.test( value );
            }, "Please specify a valid phone number." );

        },

        postInit = function (){
            if( !isLoggedIn() ) {
                // se non sono loggato alcune pagine non possono essere navigate, reindirizzo su login.html
                switch(vars.$body.data("page")){
                    case "configuration":
                    case "configuration-modem":
                    case "configuration-sensors":
                    case "configuration-remotehosts":
                    case "administration":
                    case "administration-commands":
                        window.location.href = 'login.html';
                        break;
                }
            }

            $.when( fetchNetmanData(), fetchFirmwareUpdateAvailable(), fetchNominalData(), fetchLiveData() ).always( removeLoader );

            initHeader();
            initCustomValidators();
            initCustomForms();
            initEventsHandler();
            initNotifications();

            switch(vars.$body.data("page")){
                case "dashboard":
                    Dashboard.init();
                    Dashboard.setUPSio_type(vars.nominalData.ups_type);
                    break;

                case "monitor":
                    Monitor.init();
                    break;

                case "tabledata":
                    TableData.init();
                    break;

                case "configuration":
                case "configuration-remotehosts":
                case "configuration-modem":
                    UpsConfig.init();
                    initTabNavigation(".inner-tab-nav");
                    break;

                case "configuration-sensors":
                    initTabNavigation(".inner-tab-nav");
                    Sensors.init();
                    break;

                case "history":
                    initTabNavigation(".nav-tabs");
                    Logs.init();
                    break;


                case "administration":
                    UpsConfig.init();
                    initTabNavigation(".inner-tab-nav");
                    initForms();
                    break;

                case "administration-commands":
                    UpsConfig.init();
                    break;

                case "login":
                    initLogin();
                    break;

                case "system-overview":
                    initDownloadButton();
                    break;

                case "recoverpassword":
                    initRecoveryPassword();
                    break;
            }

            vars.nominalDataFetchInterval = window.setInterval(fetchNominalData, Config.get("NominalDatarefreshInterval"));
            vars.liveDataFetchInterval = window.setInterval(fetchLiveData, Config.get("LiveDatarefreshInterval"));
            vars.netmanDataFetchInterval = window.setInterval(fetchNetmanData, Config.get("NetmanDatarefreshInterval"));
            if( isLoggedIn() ) {
                vars.sectionDataFetchInterval = window.setInterval(fetchSectionData, Config.get("RefreshTokenrefreshInterval"));
            }
        },

    init = function init(){
        setLocale('en');

        // Dropzone.js File upload
        Dropzone.autoDiscover = false;

        $.when( fetchNominalData(), fetchSectionData() ).always( postInit );
    };


    return {
        init: init,
        postInit: postInit,
        reboot: function(sec){ rebootModal(sec || 180); },
        clearIntervals : clearIntervals,
        isLoggedIn : isLoggedIn,
        whoIsLoggedIn : whoIsLoggedIn,
        showWhoisLoggedIn : showWhoisLoggedIn,
        OSVersion: OSVersion,
        isMonoMono: isMonoMono,
        isTriMono: isTriMono,
        isTriTri: isTriTri,
        isATS: isATS,
        setLastSaveConfigButton: setLastSaveConfigButton
    }
}();

Main.init();

// Codice per intercettare la chiusura/il cambio pagina per forzare implicitamente il logout.
var validNavigation = false;

function wireUpEvents() {

    function goodbye(e) {
        if (!validNavigation) {
            // cancello tutti i token ed avviso server di invalidare tutte le connessioni web
            var logout = $.ajax({
                url: 'cgi-bin/logout.cgi',
                async: false
            }).done(function(data){
                // non gestisco risposta volutamente, sto chiudendo la pagina...
                Cookies.remove('token');
                Cookies.remove("rebootRequest");
                Cookies.remove("notifications");
            });
        }
    }

    window.onbeforeunload=goodbye;

    // Attach the event click for all links in the page
    $("a").bind("click", function() {
        validNavigation = true;
    });

    // Attach the event keypress to exclude the F5 refresh
    $(document).bind("keydown", function (e) {
        if (e.keyCode == 116){
            validNavigation = true;
        }
    });

    // Attach the event submit for all forms in the page
    $("form").bind("submit", function() {
        validNavigation = true;
    });

    // Attach the event click for all inputs in the page
    $("input[type=submit]").bind("click", function() {
        validNavigation = true;
    });

}

// Wire up the events as soon as the DOM tree is ready
$(document).ready(function() {
    wireUpEvents();
});


$(document).ready(function(){
    $('[data-toggle="tooltip"]').tooltip({html: true});
});
